#!/usr/bin/bash

_jp_jdircand_arr=()
_jp_jdir_arr=()
_jp_jver_arr=()

function check_java_version() {
  local f
  for f in "${_jp_jdircand_arr[@]}"; do
    [ "$f" = "$1" ] && return 1
  done
  _jp_jdircand_arr+=("$1")
  local jd="$1"
  local jexe="$jd/bin/java"
  if [ ! -x "$jexe" ]; then
    jd="$jd/jre"
    jexe="$jd/bin/java"
    [ ! -x "$jexe" ] && return 1
  fi
  local ver
  ver=$($jexe -version 2>&1)
  if echo "$ver" | head -n 1 | grep -Eiq '(java|openjdk).* (\"){0,1}(11\.|1\.8)'; then
    _jp_jdir_arr+=("$jd")
    ver="$(echo "$ver" | sed '2q;d' | sed 's/.*(build \(.*\).*)/\1/')"
    _jp_jver_arr+=("$ver")
    echo "$ver"
    return 0
  fi
  return 1
}

function traverse_dir() {
  [ ! -d "$1" ] && return 1
  local f jd
  for f in "$1"/*; do
    case "$f" in
      *java* | *jdk* | *jre*)
        jd=$(readlink -f "$f")
        case "$jd" in
          *1.8* | *11*)
            if [ -d "$jd" ] && check_java_version "$jd" >/dev/null; then
              continue
            fi
            if [ -f "$jd" ] && [ -x "$jd" ]; then
              jd=$(dirname "$(dirname "$jd")")
              if check_java_version "$jd" >/dev/null; then
                continue
              fi
            fi
            ;;
          *) ;;
        esac
        ;;
      *) ;;
    esac
  done
}

function main() {
  local ver
  if [ -n "$JAVA_HOME" ]; then
    ver=$(check_java_version "$JAVA_HOME")
    if [ -n "$ver" ]; then
      ver=$(echo "$ver" | cut -d'.' -f1)
      [ "$ver" = "1" ] && ver=8
      echo "${JAVA_HOME}:${ver}"
      return 0
    fi
  fi
  local jexe
  jexe=$(command -v java)
  if [ -n "$jexe" ]; then
    local jd
    jd=$(dirname "$(dirname "$(readlink -f "$jexe")")")
    check_java_version "$jd" >/dev/null
  fi
  traverse_dir "/usr/lib/jvm"
  traverse_dir "/usr/java"
  traverse_dir "/etc/alternatives"

  local i maj min dir curMaj=0 curMin=0
  for i in "${!_jp_jver_arr[@]}"; do
    ver="${_jp_jver_arr[i]}"
    maj=$(echo "$ver" | cut -d'.' -f1)
    if [ "$maj" = "11" ]; then
      min=$(echo "$ver" | cut -d'.' -f3 | cut -d'+' -f1)
    else
      min=$(echo "$ver" | cut -d'_' -f2 | cut -d'-' -f1)
    fi
    if [[ "$maj" -gt "$curMaj" || ("$maj" -eq "$curMaj" && "$min" -gt "$curMin") ]]; then
      curMaj="$maj"
      curMin="$min"
      dir="${_jp_jdir_arr[i]}"
    fi
  done
  [ "$curMaj" = "1" ] && curMaj=8
  echo "${dir}:${curMaj}"
}

main "$@"
unset _jp_jdircand_arr
unset _jp_jdir_arr
unset _jp_jver_arr
exit 0
